<?php
/**
 * @file
 * This file contains code for the profile.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Allows the profile to alter the site configuration form.
 */
if (!function_exists("system_form_install_configure_form_alter")) {

  /**
   * Implements hook_form_FORM_ID_alter().
   */
  function system_form_install_configure_form_alter(&$form, $form_state) {
    $form['site_information']['site_name']['#default_value'] = 'OpenLucius';
  }

}

/**
 * Implements hook_form_alter().
 * Select the current install profile by default.
 */
if (!function_exists("system_form_install_select_profile_form_alter")) {

  /**
   * Implements hook_form_FORM_ID_alter().
   */
  function system_form_install_select_profile_form_alter(&$form, $form_state) {
    $form['profile']['OpenLucius']['#value'] = 'openlucius';
  }

}

/**
 * Implements hook_install_tasks().
 */
function openlucius_install_tasks(&$install_state) {
  $tasks = array();

  $tasks['_openlucius_features_revert_all'] = array(
    'type' => 'normal',
  );
  $tasks['_openlucius_create_first_group']  = array(
    'type' => 'normal',
  );
  $tasks['_openlucius_node_access_rebuild'] = array(
    'type' => 'normal',
  );
  $tasks['_openlucius_core_settings_form']  = array(
    'display_name' => st('Configure Openlucius'),
    'type'         => 'form',
  );

  return $tasks;
}

/**
 * Form constructor for the core settings.
 *
 * @see _openlucius_core_settings_form_validate()
 * @see _openlucius_core_settings_form_submit()
 *
 * @ingroup forms
 */
function _openlucius_core_settings_form() {
  $form['openlucius'] = array(
    '#type'   => 'fieldset',
    '#title'  => st('Openlucius specific settings'),
    '#weight' => -1000,
  );

  // Any visible, writeable wrapper can potentially be used for the files
  // directory, including a remote file system that integrates with a CDN.
  foreach (file_get_stream_wrappers(STREAM_WRAPPERS_LOCAL_NORMAL) as $scheme => $info) {
    $options[$scheme] = check_plain($info['description']);
  }

  // As Drupal hides the private field until 'file_private_path' is set
  // so we add it manually.
  // @see system_file_system_settings().
  $options['private'] = st('Private local files served by Drupal.');

  if (!empty($options)) {
    $form['openlucius']['file_default_scheme'] = array(
      '#type'          => 'radios',
      '#title'         => st('Default download method'),
      '#default_value' => isset($options['private']) ? 'private' : key($options),
      '#options'       => $options,
      '#description'   => st('This setting is used as the preferred download method.') . '<br />' . st('It is advisable to use private for a closed system such as Openlucius.'),
    );
  }

  $form['openlucius']['file_public_path'] = array(
    '#type'          => 'textfield',
    '#title'         => st('Public file system path'),
    '#default_value' => variable_get('file_public_path', conf_path() . '/files'),
    '#maxlength'     => 255,
    '#description'   => st('A local file system path where public files will be stored. This directory must exist and be writable by Drupal. This directory must be relative to the Drupal installation directory and be accessible over the web.'),
    '#required'      => TRUE,
  );

  $form['openlucius']['file_private_path'] = array(
    '#type'          => 'textfield',
    '#title'         => st('Private file system path'),
    '#default_value' => variable_get('file_private_path', ''),
    '#maxlength'     => 255,
    '#description'   => st('An existing local file system path for storing private files. It should be writable by Drupal and not accessible over the web. See the online handbook for <a href="@handbook">more information about securing private files</a>.', array('@handbook' => 'http://drupal.org/documentation/modules/file')),
  );

  $form['openlucius']['file_temporary_path'] = array(
    '#type'          => 'textfield',
    '#title'         => st('Temporary directory'),
    '#default_value' => variable_get('file_temporary_path', file_directory_temp()),
    '#maxlength'     => 255,
    '#description'   => st('A local file system path where temporary files will be stored. This directory should not be accessible over the web.'),
    '#required'      => TRUE,
  );

  $form['openlucius']['submit'] = array(
    '#type' => 'submit',
    '#value' => st('Continue'),
  );

  return $form;
}

/**
 * Form validation for settings in profile.
 * @see _openlucius_core_settings_form_submit()
 * @see _openlucius_install_check_directory()
 */
function _openlucius_core_settings_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  form_set_error(NULL, t('The directory %directory does not exist and could not be created.'));

  // Only if the scheme is set to private check that element.
  if ($values['file_default_scheme'] == 'private') {
    _openlucius_install_check_directory($form_state, 'file_private_path');
  }

  // Check public and temporary paths.
  _openlucius_install_check_directory($form_state, 'file_public_path');
  _openlucius_install_check_directory($form_state, 'file_temporary_path');
}

/**
 * Function to verify the directory and apply the correct htaccess.
 */
function _openlucius_install_check_directory(&$form_state, $element) {
  // Fetch values.
  $values = $form_state['values'];

  // Fetch the directory.
  $directory = isset($values[$element]) ? $values[$element] : '';

  // Check if path empty.
  if (strlen($values[$element]) == 0) {
    form_set_error($element, t('The directory %directory does not exist and could not be created.', array('%directory' => $directory)));
  }

  // If the directory does not exists and cannot be created.
  if (!is_dir($directory) && !drupal_mkdir($directory, NULL, TRUE)) {
    form_set_error($element, t('The directory %directory does not exist and could not be created.', array('%directory' => $directory)));
  }

  if (is_dir($directory) && !is_writable($directory) && !drupal_chmod($directory)) {
    // If the directory is not writable and cannot be made so.
    form_set_error($element, t('The directory %directory exists but is not writable and could not be made writable.', array('%directory' => $directory)));
  }
  elseif (is_dir($directory)) {

    // Add the htaccess file.
    if ($element == 'file_public_path') {

      // Create public .htaccess file.
      file_create_htaccess($directory, FALSE);
    }
    else {

      // Create private .htaccess file.
      file_create_htaccess($directory);
    }
  }
}

/**
 * Form submission handler.
 * @see _openlucius_core_settings_form_validate()
 */
function _openlucius_core_settings_form_submit($form, &$form_state) {
  $values = $form_state['values'];

  // Store the file settings.
  variable_set('file_default_scheme', $values['file_default_scheme']);
  variable_set('file_public_path', $values['file_public_path']);
  variable_set('file_private_path', $values['file_private_path']);
  variable_set('file_temporary_path', $values['file_temporary_path']);
}

/**
 * Custom function to revert all features.
 */
function _openlucius_features_revert_all() {
  drupal_set_time_limit(0);
  features_revert(array(
    'openlucius_core'  => array('field_base'),
    'openlucius_core2' => array('field_base', 'field_instance'),
  ));
  features_revert();
}

/**
 * Custom function to create first group.
 */
function _openlucius_create_first_group() {
  $node           = new stdClass();
  $node->type     = "ol_group";
  $node->title    = "First Group, Yihaaa!";
  $node->uid      = 1;
  $node->language = LANGUAGE_NONE;

  node_object_prepare($node);

  $node = node_submit($node);
  node_save($node);
}

/**
 * Custom function to node_access_rebuild.
 */
function _openlucius_node_access_rebuild() {
  node_access_rebuild();
  // Ignore any rebuild messages.
  node_access_needs_rebuild(FALSE);
  // Ignore any other install messages.
  drupal_get_messages();
}
